from kubernetes import client as k8s
from kubernetes.client.models.v1_job import V1Job

from . import Resource


class Job(Resource):
    def delete(self, body: k8s.V1DeleteOptions):
        return k8s.BatchV1Api(self.client).delete_namespaced_job(
            body=body, namespace=self.namespace, name=self.name
        )

    def create(self) -> V1Job:
        return k8s.BatchV1Api(self.client).create_namespaced_job(
            body=self.resource, namespace=self.namespace
        )

    def status(self) -> V1Job:
        return k8s.BatchV1Api(self.client).read_namespaced_job_status(
            name=self.name, namespace=self.namespace
        )



from kubernetes import client as k8s

from . import Resource


class Deployment(Resource):
    def delete(self, body: k8s.V1DeleteOptions):
        return k8s.AppsV1Api(self.client).delete_namespaced_deployment(
            body=body, namespace=self.namespace, name=self.name
        )

    def create(self):
        return k8s.AppsV1Api(self.client).create_namespaced_deployment(
            body=self.resource, namespace=self.namespace
        )

    def status(self):
        return k8s.AppsV1Api(self.client).read_namespaced_stateful_set_status(
            name=self.name, namespace=self.namespace
        )



import base64
import re
import typing as t
from pathlib import Path

import yaml
from kubernetes import client as k8s
from kubernetes import config
from passlib import pwd

from tulips.resource import Resource, ResourceRegistry


class Tulip:
    def __init__(
        self,
        conf: str,
        namespace: str,
        meta: t.Dict,
        spec_path: str,
        override: str = None,
    ) -> None:
        """Manages deployment.

        Args:
            conf (str): Path to Kubernetes config.
            namespace (str): Kubernetes namespace.
            meta (t.Dict): Spec variables.
            spec_path (str): Location of chart to deploy.
            override (str): Override specific resource file, defaults to
             `resource.override.yaml` where one has `resource.yaml`.
        """

        self.client: k8s.ApiClient = config.new_client_from_config(conf)

        # Disable threadding pool inside swagger client
        # because it causes problems with other multithreading workers
        # as we don't use any async calls we are safe to do so.
        self.client.pool.close()
        self.client.pool.join()

        self.meta = meta
        self.namespace = namespace
        self.override = override if override else "override"
        self.spec_path = spec_path

    def create_namespace(self) -> k8s.V1NamespaceStatus:
        """Create namespace.

        Returns:
            k8s.V1NamespaceStatus
        """

        body = k8s.V1Namespace(
            metadata=k8s.V1ObjectMeta(
                name=self.namespace, labels={"store_id": self.namespace}
            )
        )
        return k8s.CoreV1Api(self.client).create_namespace(body)

    def delete_namespace(self) -> k8s.V1NamespaceStatus:
        """Delete namespace.

        Returns:
            k8s.V1NamespaceStatus
        """
        body = k8s.V1DeleteOptions(
            propagation_policy="Foreground", grace_period_seconds=5
        )
        return k8s.CoreV1Api(self.client).delete_namespace(
            self.namespace, body
        )

    def resources(self) -> t.Iterator[Resource]:
        """Deployment specification.

        Returns:
            t.Iterator[Resource]: Iterator over specifications
        """

        pattern = re.compile(r"^(.*)<%=(?:\s+)?(\S*)(?:\s+)?=%>(.*)$")
        yaml.add_implicit_resolver("!meta", pattern)
        maps = {"@pwd": lambda: base64.b64encode(pwd.genword(length=24))}
        maps.update(self.meta)

        def meta_constructor(loader, node):
            value = loader.construct_scalar(node)
            start, name, end = pattern.match(value).groups()
            val = maps[name]
            if name.startswith("@"):
                val = val()
            return start + val + end

        yaml.add_constructor("!meta", meta_constructor)
        path = Path(self.spec_path).joinpath("templates")

        for res in path.glob("*.yaml"):
            base_name = str(res.name)[:-5]  # strip .yaml
            override = res.parent.joinpath(
                "overrides", f"{base_name}.{self.override}.yaml"
            )
            # use override if it is found else use original file
            source_file = override if override.is_file() else res

            text = self.prepare(source_file)
            for spec in yaml.load_all(text):
                yield ResourceRegistry.get_cls(spec["kind"])(
                    self.client, self.namespace, spec, source_file
                )

    def prepare(self, f: Path) -> str:
        """Yaml parser trips at {{}} so we replace it with custom constructor.

        Arguments:
            f {Path} -- Path instance.

        Returns:
            str -- Updated yaml string of the resource.
        """

        return f.read_text().replace("{{", "<%=").replace("}}", "=%>")



import abc

from kubernetes import client as k8s
import typing as t


class UndefinedResource(Exception):
    """Resource was not defined error."""

    def __init__(self, kind: str) -> None:
        self.str = kind
        Exception.__init__(
            self, f"{kind} is not yet defined[{ResourceRegistry.REGISTRY}]"
        )


class ResourceRegistry(type):

    REGISTRY: dict = {}

    def __new__(cls, name, bases, attrs):
        new_cls = type.__new__(cls, name, bases, attrs)
        cls.REGISTRY[new_cls.__name__] = new_cls
        return new_cls

    @classmethod
    def get_cls(cls, kind: str) -> t.Callable:
        """Get Class for provided kind.

        Args:
            kind (str): Name of the kind that implements Resource.

        Raises:
            UndefinedResource: Resource class for provided kind is not defined.

        Returns:
            t.Callable: [Resource] class that implements resource.
        """

        kind_cls = cls.REGISTRY.get(kind)
        if not kind_cls:
            raise UndefinedResource(kind)
        return kind_cls


class Resource(metaclass=ResourceRegistry):
    """Resource is interface that describes Kubernetes Resource defintion."""

    resource: dict
    client: k8s.ApiClient
    namespace: str

    def __init__(
        self,
        client: k8s.ApiClient,
        namespace: str,
        resource: dict,
        source_file: str,
    ) -> None:
        """Initializes resource or CRD.

        Args:
            client (k8s.ApiClient): Instance of the Kubernetes client.
            namespace (str): Namespace where Workload should be deployed.
            resource (dict): Kubernetes resource or CRD.
            source_file (str): Source file from which this resource was
              generated from.
        """

        self.client = client
        self.namespace = namespace
        self.resource = resource
        self.source_file = resource

    @abc.abstractmethod
    def create(self):
        """Create Resource."""
        pass

    @abc.abstractmethod
    def delete(self, body: k8s.V1DeleteOptions):
        """Delete Resource."""
        pass

    @abc.abstractmethod
    def status(self):
        """Info about Resource."""
        pass

    @property
    def name(self):
        """Returns the class kind.

        Returns:
            [str]: Base name of the class and kind
        """

        return self.resource["metadata"]["name"]



from kubernetes import client as k8s

from . import Resource


class Issuer(Resource):
    """A `cert-manager` Issuer resource."""

    version = "v1alpha1"
    group = "certmanager.k8s.io"
    plural = "issuers"

    def delete(self, body: k8s.V1DeleteOptions):
        return k8s.CustomObjectsApi(
            self.client
        ).delete_namespaced_custom_object(
            body=body,
            namespace=self.namespace,
            version=self.version,
            group=self.group,
            plural=self.plural,
            name=self.name,
        )

    def create(self):
        return k8s.CustomObjectsApi(
            self.client
        ).create_namespaced_custom_object(
            body=self.resource,
            namespace=self.namespace,
            version=self.version,
            group=self.group,
            plural=self.plural,
        )

    def status(self):
        return k8s.CustomObjectsApi(self.client).get_namespaced_custom_object(
            name=self.name,
            namespace=self.namespace,
            version=self.version,
            group=self.group,
            plural=self.plural,
        )



from kubernetes import client as k8s
from kubernetes.client.models.v1beta1_cron_job import V1beta1CronJob

from . import Resource


class CronJob(Resource):
    def delete(self, body: k8s.V1DeleteOptions):
        return k8s.BatchV1beta1Api(self.client).delete_namespaced_cron_job(
            body=body, namespace=self.namespace, name=self.name
        )

    def create(self) -> V1beta1CronJob:
        return k8s.BatchV1beta1Api(self.client).create_namespaced_cron_job(
            body=self.resource, namespace=self.namespace
        )

    def status(self) -> V1beta1CronJob:
        return k8s.BatchV1beta1Api(
            self.client
        ).read_namespaced_cron_job_status(
            name=self.name, namespace=self.namespace
        )



from kubernetes import client as k8s
from kubernetes.client.models.v1_stateful_set import V1StatefulSet

from . import Resource


class StatefulSet(Resource):
    def delete(self, body: k8s.V1DeleteOptions):
        return k8s.AppsV1Api(self.client).delete_namespaced_stateful_set(
            body=body, namespace=self.namespace, name=self.name
        )

    def create(self) -> V1StatefulSet:
        return k8s.AppsV1Api(self.client).create_namespaced_stateful_set(
            body=self.resource, namespace=self.namespace
        )

    def status(self) -> V1StatefulSet:
        return k8s.AppsV1Api(self.client).read_namespaced_stateful_set_status(
            name=self.name, namespace=self.namespace
        )



import click
import structlog
import yaml
from kubernetes import client as k8s
from kubernetes.client.rest import ApiException

from .tulip import Tulip

log = structlog.get_logger("tulip")

__version__ = "0.0.3"


@click.group()
def cli():
    pass


@click.command(
    context_settings=dict(ignore_unknown_options=True, allow_extra_args=True),
    help=(
        "You can pass chart variables via foo=bar, "
        "for example '$ tulip push app.yaml foo=bar'"
    ),
)
@click.argument("chart", type=click.Path(exists=True))
@click.option("--namespace", default="default", help="Kubernetes namespace")
@click.option("--release", help="Name of the release")
@click.option(
    "--kubeconfig",
    help="Path to kubernetes config",
    default="$HOME/.kube/config",
    type=click.Path(exists=True),
)
@click.pass_context
def push(ctx, chart, namespace, release, kubeconfig):
    options = {
        ".Release.Name": release,
        "namespace": namespace,
        "chart": chart,
    }
    for item in ctx.args:
        options.update([item.split("=")])
    click.echo(options)
    client = Tulip(kubeconfig, namespace, options, chart)
    for resource in client.resources():
        try:
            resource.create()
            log.info(
                "Created",
                name=resource.name,
                Resource=resource.__class__.__name__,
            )
        except ApiException as e:
            log.error(
                "Failed creating",
                name=resource.name,
                Resource=chart.__class__.__name__,
                reason=e.reason,
            )


@click.command(
    context_settings=dict(ignore_unknown_options=True, allow_extra_args=True),
    help=(
        "You can pass chart variables via foo=bar, "
        "for example '$ tulip push app.yaml foo=bar'"
    ),
)
@click.argument("chart", type=click.Path(exists=True))
@click.option("--namespace", default="default", help="Kubernetes namespace")
@click.option("--release", help="Name of the release")
@click.option(
    "--kubeconfig",
    help="Path to kubernetes config",
    default="$HOME/.kube/config",
    type=click.Path(exists=True),
)
@click.pass_context
def status(ctx, chart, namespace, release, kubeconfig):
    options = {
        ".Release.Name": release,
        "namespace": namespace,
        "chart": chart,
    }
    for item in ctx.args:
        options.update([item.split("=")])
    client = Tulip(kubeconfig, namespace, options, chart)
    for resource in client.resources():
        try:
            click.echo(resource.status().status)
        except ApiException as e:
            log.error(
                "Failed getting info",
                name=resource.name,
                Resource=chart.__class__.__name__,
                reason=e.reason,
            )


@click.command(
    context_settings=dict(ignore_unknown_options=True, allow_extra_args=True),
    help=(
        "You can pass chart variables via foo=bar, "
        "for example '$ tulip push app.yaml foo=bar'"
    ),
)
@click.argument("chart", type=click.Path(exists=True))
@click.option("--namespace", default="default", help="Kubernetes namespace")
@click.option("--release", help="Name of the release")
@click.option(
    "--kubeconfig",
    help="Path to kubernetes config",
    default="$HOME/.kube/config",
    type=click.Path(exists=True),
)
@click.pass_context
def echo(ctx, chart, namespace, release, kubeconfig):
    options = {
        ".Release.Name": release,
        "namespace": namespace,
        "chart": chart,
    }
    for item in ctx.args:
        options.update([item.split("=")])
    click.echo(f"### meta={options}")
    client = Tulip(kubeconfig, namespace, options, chart)
    for resource in client.resources():
        click.echo(yaml.dump(resource.resource))
        click.echo("---")


@click.command(
    context_settings=dict(ignore_unknown_options=True, allow_extra_args=True),
    help=(
        "You can pass chart variables via foo=bar, "
        "for example '$ tulip rm app.yaml foo=bar'"
    ),
)
@click.argument("chart", type=click.Path(exists=True))
@click.option("--namespace", default="default", help="Kubernetes namespace")
@click.option("--release", help="Name of the release")
@click.option(
    "--kubeconfig",
    help="Path to kubernetes config",
    type=click.Path(exists=True),
)
@click.pass_context
def rm(ctx, chart, namespace, release, kubeconfig):
    options = {"release": release, "namespace": namespace, "chart": chart}
    for item in ctx.args:
        options.update([item.split("=")])
    click.echo(options)
    client = Tulip(kubeconfig, namespace, options, chart)
    delete = k8s.V1DeleteOptions(
        propagation_policy="Foreground", grace_period_seconds=5
    )
    for chart in client.resources():
        try:
            chart.delete(body=delete)
            log.info(
                "Deleted", name=chart.name, Resource=chart.__class__.__name__
            )
        except ApiException as e:
            log.error(
                "Failed deleting",
                name=chart.name,
                Resource=chart.__class__.__name__,
                reason=e.reason,
            )


def main():
    cli.add_command(rm)
    cli.add_command(push)
    cli.add_command(echo)
    cli.add_command(status)
    cli()


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['tulips', 'tulips.resource']

package_data = \
{'': ['*']}

install_requires = \
['PyYAML>=3.12,<4.0',
 'click>=6.7,<7.0',
 'kubernetes==7.0',
 'passlib>=1.7,<2.0',
 'structlog>=18.1,<19.0']

setup_kwargs = {
    'name': 'tulips',
    'version': '1.2.0',
    'description': 'Wrapper around kubernetes-clients/python',
    'long_description': '[![Build Status](https://travis-ci.com/dz0ny/tulips.svg?branch=master)](https://travis-ci.com/dz0ny/tulips)\n[![Code style: black](https://img.shields.io/badge/code%20style-black-000000.svg)](https://github.com/ambv/black)\n[![Type checker: mypy](https://img.shields.io/badge/type%20checker-mypy-1F5082.svg)](https://github.com/python/mypy)\n[![Packaging: poetry](https://img.shields.io/badge/packaging-poetry-C2CAFD.svg)](https://poetry.eustace.io/)\n[![Packaging: poetry](https://img.shields.io/badge/packaging-pypi-006dad.svg)](https://pypi.org/project/tulips/)\n[![codecov](https://codecov.io/gh/dz0ny/tulips/branch/master/graph/badge.svg)](https://codecov.io/gh/dz0ny/tulips)\n\n# Tulips\n\nA small wrapper around https://github.com/kubernetes-client/python which understands Kubernetes charts.\n\n## Why\n\nI needed something simple that would read Helm charts and push them to the Kubernetes cluster and\nbe extensible. So something like helm+kubectl with ability to write you own tools around them.\n\n## Supported CRDS aka Kubernetes resources\n\n- Deployment\n- Service\n- Ingress\n- Secret\n- Issuer (cert-manager)\n- PersistentVolumeClaim\n\n## Example use\n\n```python\n\nimport yaml\nfrom tulips.resources import ResourceRegistry\nfrom kubernetes import client as k8s\nfrom kubernetes import config\n\n\nclient = config.new_client_from_config(\'kube.conf\')\n\nspec = yaml.load(\'ingress.yaml\')\n\ningress_cls = ResourceRegistry.get_cls(spec[\'kind\'])\ningress = ingress_cls(config.client, namespace=\'default\', spec)\ningress.create()  # Create Ingress resource\ningress.delete()  # Delete Ingress resource\n```\n\n## Adding new resource\n\nIn order to add support for new Kubernetes resource, one needs to create class\nthat inherits from `tulips.resources.Resource` class.\n\n### Example resource\n\n```python\nimport tulips.resources.Resource\n\nclass ClusterIssuer(Resource):\n    """A `cert-manager` ClusterIssuer resource."""\n\n    version = "v1alpha1"\n    group = "certmanager.k8s.io"\n    plural = "clusterissuers"\n\n    def delete(self, body: k8s.V1DeleteOptions):\n        return k8s.CustomObjectsApi(\n            self.client\n        ).delete_namespaced_custom_object(\n            body=body,\n            namespace=self.namespace,\n            version=self.version,\n            group=self.group,\n            plural=self.plural,\n            name=self.name,\n        )\n\n    def create(self):\n        return k8s.CustomObjectsApi(\n            self.client\n        ).create_namespaced_custom_object(\n            body=self.resource,\n            namespace=self.namespace,\n            version=self.version,\n            group=self.group,\n            plural=self.plural,\n        )\n```\n\nIt will be registered into the `ResourceRegistry` and can be fetched via `ResourceRegistry.get_cls` method.\n\n## Tulip\n\nTulip is a sample client that emulates Helm but without `tiller`.\n\n```shell\n$ python tulips push --help                                    06/25/18 -  9:49\nUsage: tulips push [OPTIONS] CHART\n\n  You can pass chart variables via foo=bar, for example \'$ tulip push\n  app.yaml foo=bar\'\n\nOptions:\n  --namespace TEXT   Kubernetes namespace\n  --release TEXT     Name of the release\n  --kubeconfig PATH  Path to kubernetes config\n  --help             Show this message and exit.\n\n```\n\n### Example client\n\nLet\'s say that I want to deploy a Secret and Ingress\n\n```yaml\napiVersion: v1\nkind: Secret\nmetadata:\n  name: {{ release }}-secrets\ntype: Opaque\ndata:\n  password: {{ @pwd }}\n---\napiVersion: extensions/v1beta1\nkind: Ingress\nmetadata:\n  name: {{ release }}-web-ingress\n  labels:\n    app: woocart-{{ release }}\n  annotations:\n    nginx.ingress.kubernetes.io/limit-connections: "100"\n    kubernetes.io/ingress.class: nginx\nspec:\n  rules:\n  - host: {{ domain }}\n    http:\n      paths:\n        - path: /\n          backend:\n            serviceName: {{ release }}-web\n            servicePort: 80\n```\n\nIf one runs `tulip --release test push --kubeconf kube.conf app.yaml domain=test.tld\'\n\nSpec file is inspected and all `{{ variables }}` are replaced with real values. Also\nspecial `{{ @pwd }}` will generate strong password using `passlib` library.\n',
    'author': 'Janez Troha',
    'author_email': 'dz0ny@ubuntu.si',
    'url': 'https://github.com/dz0ny/tulips',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)



from tulips.resource.deployment import Deployment  # noqa: F401
from tulips.resource.ingress import Ingress  # noqa: F401
from tulips.resource.issuer import Issuer  # noqa: F401
from tulips.resource.persistentvolumeclaim import (  # noqa: F401
    PersistentVolumeClaim,
)
from tulips.resource.secret import Secret  # noqa: F401
from tulips.resource.service import Service  # noqa: F401
from tulips.resource.statefullset import StatefulSet  # noqa: F401
from tulips.resource.configmap import ConfigMap  # noqa: F401
from tulips.resource.cronjob import CronJob  # noqa: F401
from tulips.resource.job import Job  # noqa: F401
